==> scandit_flutter_datacapture_parser-6.19.8.tar.gz: lib/src/parser.dart <==
/*
 * This file is part of the Scandit Data Capture SDK
 *
 * Copyright (C) 2021- Scandit AG. All rights reserved.
 */

import 'dart:async';
import 'dart:convert';

import 'package:flutter/services.dart';
import 'package:scandit_flutter_datacapture_core/scandit_flutter_datacapture_core.dart';
import 'function_names.dart';
import 'parsed_data.dart';
import 'parser_dataformat.dart';

class Parser extends DataCaptureComponent implements Serializable {
  String get id => super.id;

  late _ParserController _controller;

  late ParserDataFormat _dataFormat;

  final Map<String, dynamic> _options = {};

  late DataCaptureContext _context;

  static Future<Parser> forContextAndFormat(DataCaptureContext context, ParserDataFormat dataFormat) {
    var parser = Parser._();
    parser._dataFormat = dataFormat;
    parser._context = context;
    var future = context.addComponent(parser).then((_) => parser);
    return future;
  }

  Future<void> setOptions(Map<String, dynamic> options) {
    _options.clear();
    _options.addAll(options);
    return _context.update();
  }

  Future<ParsedData> parseString(String data) {
    return _controller.parseString(data);
  }

  Future<ParsedData> parseRawData(String data) {
    return _controller.parseRawData(data);
  }

  Parser._() : super(DateTime.now().toUtc().millisecondsSinceEpoch.toString()) {
    _controller = _ParserController(this);
  }

  @override
  Map<String, dynamic> toMap() {
    var json = super.toMap();
    json.addAll({'type': 'parser', 'dataFormat': _dataFormat.toString(), 'options': _options});
    return json;
  }
}

class _ParserController {
  final Parser _parser;

  final MethodChannel _methodChannel = MethodChannel('com.scandit.datacapture.parser.method/parser');

  _ParserController(this._parser);

  Future<ParsedData> parseString(String data) {
    var arguments = _createParserInvocationArgs(data);
    return _methodChannel
        .invokeMethod(FunctionNames.parseStringMethodName, arguments)
        .then(_parseData, onError: _onError);
  }

  Future<ParsedData> parseRawData(String data) {
    var arguments = _createParserInvocationArgs(data);
    return _methodChannel
        .invokeMethod(FunctionNames.parseRawDataMethodName, arguments)
        .then(_parseData, onError: _onError);
  }

  Map<String, dynamic> _createParserInvocationArgs(String data) {
    return {'parserId': _parser.id, 'data': data};
  }

  FutureOr<ParsedData> _parseData(dynamic result) {
    var jsonData = jsonDecode(result) as List<dynamic>;
    var json = jsonData.cast<Map<String, dynamic>>();
    var parsedData = ParsedData.fromJSON(json);
    return parsedData;
  }

  void _onError(Object? error, StackTrace? stackTrace) {
    if (error == null) return;
    print(error);

    if (stackTrace != null) {
      print(stackTrace);
    }

    throw error;
  }
}

==> scandit_flutter_datacapture_parser-6.22.0.tar.gz: lib/src/parser.dart <==
/*
 * This file is part of the Scandit Data Capture SDK
 *
 * Copyright (C) 2021- Scandit AG. All rights reserved.
 */

import 'dart:async';
import 'dart:convert';

import 'package:flutter/services.dart';
import 'package:scandit_flutter_datacapture_core/scandit_flutter_datacapture_core.dart';
import 'function_names.dart';
import 'parsed_data.dart';
import 'parser_dataformat.dart';

class Parser extends DataCaptureComponent implements Serializable {
  String get id => super.id;

  late _ParserController _controller;

  final ParserDataFormat _dataFormat;

  final Map<String, dynamic> _options = {};

  // ignore: unused_field
  final DataCaptureContext _context;

  static Future<Parser> forContextAndFormat(DataCaptureContext context, ParserDataFormat dataFormat) {
    var parser = Parser._(context, dataFormat);
    return parser._controller.createUpdateNativeInstance().then((value) => parser);
  }

  Future<void> setOptions(Map<String, dynamic> options) {
    _options.clear();
    _options.addAll(options);
    return _controller.createUpdateNativeInstance();
  }

  Future<ParsedData> parseString(String data) {
    return _controller.parseString(data);
  }

  Future<ParsedData> parseRawData(String data) {
    return _controller.parseRawData(data);
  }

  Parser._(this._context, this._dataFormat) : super(DateTime.now().toUtc().millisecondsSinceEpoch.toString()) {
    _controller = _ParserController(this);
  }

  void dispose() {
    _controller.dispose();
  }

  @override
  Map<String, dynamic> toMap() {
    var json = super.toMap();
    json.addAll({'type': 'parser', 'dataFormat': _dataFormat.toString(), 'options': _options});
    return json;
  }
}

class _ParserController {
  final Parser _parser;

  final MethodChannel _methodChannel = MethodChannel('com.scandit.datacapture.parser/method_channel');

  _ParserController(this._parser);

  Future<void> createUpdateNativeInstance() {
    var encoded = jsonEncode(this._parser.toMap());
    return _methodChannel.invokeMethod(FunctionNames.createUpdateNativeInstance, encoded).onError(_onError);
  }

  Future<ParsedData> parseString(String data) {
    var arguments = _createParserInvocationArgs(data);
    return _methodChannel
        .invokeMethod(FunctionNames.parseStringMethodName, jsonEncode(arguments))
        .then(_parseData, onError: _onError);
  }

  Future<ParsedData> parseRawData(String data) {
    var arguments = _createParserInvocationArgs(data);
    return _methodChannel
        .invokeMethod(FunctionNames.parseRawDataMethodName, jsonEncode(arguments))
        .then(_parseData, onError: _onError);
  }

  Map<String, dynamic> _createParserInvocationArgs(String data) {
    return {'parserId': _parser.id, 'data': data};
  }

  FutureOr<ParsedData> _parseData(dynamic result) {
    var jsonData = jsonDecode(result) as List<dynamic>;
    var json = jsonData.cast<Map<String, dynamic>>();
    var parsedData = ParsedData.fromJSON(json);
    return parsedData;
  }

  void _onError(Object? error, StackTrace? stackTrace) {
    if (error == null) return;
    print(error);

    if (stackTrace != null) {
      print(stackTrace);
    }

    throw error;
  }

  void dispose() {
    _methodChannel.invokeMethod(FunctionNames.disposeParser, _parser.id).onError(_onError);
  }
}
